/*
 * convkye: Convert Kye92 levels to SKye format
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "skye.h"

void convert_level(const char *filename,const char *outfile,const char *author);
int convert_tile(int tile);
void fwritestr(const char *str, FILE *fp);
void reverse_endian_long(long *L);
void reverse_endian_short(short *S);

//{{{
int main(int argc, char **argv)
{
	int ii;
	const char *outfilename = NULL, *authorname=NULL;
	
	for(ii=1; ii<argc; ii++)
	{
		if(!strcmp(argv[ii], "--help") || !strcmp(argv[ii], "-h")) {
			printf("Usage:\n"
				   "    %s [-h] [-v] -a [author] -o [output] [file]\n",argv[0]);
		} else if(!strcmp(argv[ii], "-v") || !strcmp(argv[ii], "--version")) {
			printf(
"convkye 1.0, by Jim Babcock. This program is distributed under the GNU General"
"Public License; see the file COPYING for details.");
		} else if(!strcmp(argv[ii], "-o")) {
			if( ++ii >= argc ) {
				fprintf(stderr, "-o option must be followed by a filename.\n");
				exit(1);
			} else {
				outfilename = argv[ii];
			}
		} else if(!strcmp(argv[ii], "-a")) {
			if( ++ii >= argc ) {
				fprintf(stderr, "-a option must be followed by a name.\n");
				exit(1);
			} else {
				authorname = argv[ii];
			}
		} else {
			if(!outfilename) {
				fprintf(stderr, "No output file specified.\n");
				exit(1);
			}
			if(!authorname) {
				fprintf(stderr,
					"Warning: No author specified. Using 'anonymous'.\n");
				authorname="Anonymous";
			}
			convert_level(argv[ii], outfilename, authorname);
		}
	}
	return 0;
}
//}}}
//{{{
void convert_level(const char *filename, const char *outfile, const char *author)
{
	char in[80];
	int ii, jj, kk;
	int num_levels;
	unsigned short offset;
	file_header header;
	map_r2 maphead;
	FILE *fin = fopen(filename, "r");
	FILE *fout = fopen(outfile, "wb");
	
	fgets(in, 80, fin); // First line should be level count
	num_levels = atoi(in);
	
	memcpy(header.magic_bytes, "SKye", 4);
	header.revision = 2;
	header.num_levels = num_levels;
	reverse_endian_long(&header.revision);
	reverse_endian_long(&header.num_levels);
	
	fwrite(&header, sizeof(header), 1, fout);
	for(ii=0; ii<2*num_levels; ii++)
		fputc(0, fout);
	
	fwritestr("Converted set", fout);
	
	maphead.width = 30;
	maphead.height = 20;
	maphead.gamerules = 1;
	memset(maphead.unused, 0, 10);
	reverse_endian_short(&maphead.width);
	reverse_endian_short(&maphead.height);
	reverse_endian_short(&maphead.gamerules);
	
	for(ii=0; ii<num_levels; ii++)
	{
		offset = ftell(fout);
		reverse_endian_short(&offset);
		fseek( fout, sizeof(header)+2*ii, SEEK_SET );
		fwrite(&offset, sizeof(offset), 1, fout);
		fseek( fout, 0, SEEK_END);
		
		fwrite(&maphead, sizeof(maphead), 1, fout);
		fgets(in, 80, fin); fwritestr(in, fout); // Map name
		fwritestr(author, fout);                 // Map author
		fgets(in, 80, fin); fwritestr(in, fout); // Map hint
		fgets(in, 80, fin); fwritestr(in, fout); // Map congrat
		
		for(jj=0; jj<20; jj++) {
			fgets(in, 80, fin);
			for(kk=0; kk<30; kk++)
				fputc( convert_tile(in[kk]), fout );
		}
	}
	
	fclose(fout);
	fclose(fin);
}
//}}}
//{{{
int convert_tile(int tile)
{
	switch(tile)
	{
		case ' ': return TILE_EMPTY;
		case '1': return TILE_WALL_1;
		case '2': return TILE_WALL_2;
		case '3': return TILE_WALL_3;
		case '4': return TILE_WALL_4;
		case '5': return TILE_WALL_5;
		case '6': return TILE_WALL_6;
		case '7': return TILE_WALL_7;
		case '8': return TILE_WALL_8;
		case '9': return TILE_WALL_9;
		case 'K': return TILE_KYE;
		case 'e': return TILE_FUZZY;
		case '*': return TILE_DIAMOND;
		case 'b': return TILE_BLOCK;
		case 'B': return TILE_BOULDER;
		case 'u': return TILE_SLIDER_N;
		case 'd': return TILE_SLIDER_S;
		case 'l': return TILE_SLIDER_W;
		case 'r': return TILE_SLIDER_E;
		case 's': return TILE_MAGNET_NS;
		case 'S': return TILE_MAGNET_EW;
		case 'U': return TILE_PUSHER_N;
		case 'D': return TILE_PUSHER_S;
		case 'L': return TILE_PUSHER_W;
		case 'R': return TILE_PUSHER_E;
		case '^': return TILE_ROCKY_N;
		case 'v': return TILE_ROCKY_S;
		case '<': return TILE_ROCKY_W;
		case '>': return TILE_ROCKY_E;
		case 'T': return TILE_TWISTER;
		case 'E': return TILE_GNASHER;
		case 'C': return TILE_BLOB;
		case '~': return TILE_VIRUS;
		case '[': return TILE_SPIKE;
		case 'a': return TILE_TWIST_CW;
		case 'c': return TILE_TWIST_CCW;
		case 'f': return DOOR_WE;
		case 'g': return DOOR_EW;
		case 'h': return DOOR_NS;
		case 'i': return DOOR_SN;
		
		case '_': return TILE_PIT;
		case '!': return TILE_EXPLODER;
		
		case 'A': // Autoslider
		case 'F': // Autorocky
		case 'H': // Blacky
		case '{': case '|': // Timer
		case '}': case 'z':
		case 'y': case 'x':
		case 'w':
		default:
			fprintf(stderr, "Warning: unsupported tile '%c' used.\n", tile);
			return TILE_EMPTY;
	}
}
//}}}

//{{{
void fwritestr(const char *str, FILE *fp)
{
	while(*str != '\0' && *str != '\n') {
		fputc(*str, fp);
		str++;
	}
	fputc('\0', fp);
}
//}}}
//{{{
void reverse_endian_long(long *L)
{
	char *C = (char*)L;
	char tmp;
	
	tmp = C[0];
	C[0] = C[3];
	C[3] = tmp;
	tmp = C[1];
	C[1] = C[2];
	C[2] = tmp;
}
//}}}
//{{{
void reverse_endian_short(short *S)
{
	char *C = (char*)S;
	char tmp;
	
	tmp = C[0];
	C[0] = C[1];
	C[1] = tmp;
}
//}}}

